import time
import threading

from .usbctrl import Control
from db import Database

def raw_to_co_lpg(raw_value):
    if(raw_value >= 70):
        return 0
    elif(raw_value < 70 and raw_value >= 60):
        return 1
    elif(raw_value < 60 and raw_value >= 50):
        return 2
    elif(raw_value < 50 and raw_value >= 35):
        return 3
    else:
        return 4


class Core:

    db = Database()

    def __init__(self, device: Control = Control(), interval=60):

        self.db.connect()
        self.db.create()
        self.device = Control()
        self.update = True
        self.interval = interval
        self.thread_start = False

    def start_update(self, silent: bool = True):
        self.thread_start = True
        if silent: print("Silent Run")
        thread = threading.Thread(target=self.get_update_from_device, args=([silent]))
        thread.daemon = True
        thread.start()

    def stop_update(self):
        self.update = False
        self.thread_start = False

    def get_update_from_device(self, silent):
        self.update = True
        while self.update:
            gas_data = self.device.read_gas(silent)
            self.db.insert(gas_data, raw_to_co_lpg(gas_data))
            time.sleep(self.interval)
        print("Stop Updating")
        return 0

if '__main__' == __name__:

    task = Core()
    task.start_update()
    sleep(10)
    task.stop_update()
