#include<SPI.h>
#include<Wire.h>
#include<Adafruit_GFX.h>
#include<Adafruit_SSD1306.h>
#include<usbdrv.h>
#include<math.h>

/*Screen Configuration*/
#define SCREEN_WIDTH 128
#define SCREEN_HEIGHT 32

#define OLED_RESET -1
Adafruit_SSD1306 OLED(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);

/*Pin Configuration*/

#define MB_LED PIN_PD3
#define RED PIN_PD5
#define YELLOW PIN_PD1
#define GREEN PIN_PD6
#define GAS PIN_PC0
#define SDA PIN_PC4 // Don't change this pin.
#define SCL PIN_PC5 // This too.
#define BUZZER PIN_PD0

/*Request type*/

#define RQ_GET_GAS 1
#define RQ_SET_ALARM_STATE 2
#define RQ_GET_ALARM_STATE 3

/*Global Value*/
uint16_t G_gas_value;
uint8_t G_alarm_state;

/*Utility Function*/
uint16_t get_mq9_reading(uint8_t APIN){
  float sensor_voltage;
  float RS_air;
  float R0 = 0.9;
  float sensor_value=0;

  for(int i=0; i<100; i++){
    sensor_value = sensor_value + analogRead(APIN);
  }
  sensor_value /= 100;
  sensor_voltage = (sensor_value/1024)*5;

  RS_air = (5-sensor_voltage)/sensor_voltage;
  R0 = 0.87;
  float ratio = RS_air/R0;

  /*Calculate Ratio to Decimal Sca Here*/
  return round(ratio*100);
}

/*Polling Request Function*/
usbMsgLen_t usbFunctionSetup(uint8_t data[8]){
  usbRequest_t *rq =(usbRequest_t*)data;

  static uint8_t gas_value; //8 upper bit and 8 lower bit
  static uint8_t alarm_state;

  if(rq->bRequest == RQ_GET_GAS){
    gas_value = round(G_gas_value/10);
    usbMsgPtr = &gas_value;
    return 1;
  }

  else if(rq->bRequest == RQ_SET_ALARM_STATE){
    G_alarm_state = rq -> wValue.bytes[0];
    return 0;
  }

  else if(rq->bRequest == RQ_GET_ALARM_STATE){
    alarm_state = G_alarm_state;
    usbMsgPtr = &alarm_state;
    return 1;
  }

  /*Unknown Function Return*/
  return 0;
}

void setup() {

  /*Set Display Allocation*/
  if(!OLED.begin(SSD1306_SWITCHCAPVCC, 0x3C)){
    Serial.println("Allocation Failed");
  }

  /*Initialize USB-Serial Interface*/
  //Serial.begin(9600);
 
  /*Setup Pin*/
  pinMode(RED, OUTPUT);
  pinMode(YELLOW, OUTPUT);
  pinMode(GREEN, OUTPUT);
  pinMode(MB_LED, OUTPUT);
  pinMode(GAS, INPUT);
  pinMode(SDA, OUTPUT);
  pinMode(SCL, OUTPUT);
  pinMode(BUZZER, OUTPUT);

  /*OLED STARTUP*/
  OLED.clearDisplay();
  OLED.setTextColor(WHITE, BLACK);
  OLED.setCursor(0, 0);
  OLED.setTextSize(1);
  OLED.println("Kitchen Hazard");
  OLED.println("Warning System");
  OLED.setTextColor(BLACK, WHITE);
  OLED.println("Initializing");
  OLED.display();
  
  /*First time Diagnostic Run*/
  digitalWrite(RED, HIGH);
  digitalWrite(GREEN, HIGH);
  digitalWrite(YELLOW, HIGH);
  digitalWrite(BUZZER, LOW);
  delay(1000);
  OLED.clearDisplay();
  OLED.setTextColor(WHITE, BLACK);
  OLED.setCursor(0, 0);
  OLED.setTextSize(1);
  OLED.println("Kitchen Hazard");
  OLED.println("Warning System");
  OLED.setTextColor(BLACK, WHITE);
  OLED.println("Initializing");
  OLED.setTextColor(WHITE, BLACK);
  OLED.println("TEST: RED");
  OLED.display();
  digitalWrite(RED, LOW);
  delay(1000);
  OLED.clearDisplay();
  OLED.setTextColor(WHITE, BLACK);
  OLED.setCursor(0, 0);
  OLED.setTextSize(1);
  OLED.println("Kitchen Hazard");
  OLED.println("Warning System");
  OLED.setTextColor(BLACK, WHITE);
  OLED.println("Initializing");
  OLED.setTextColor(WHITE, BLACK);
  OLED.println("TEST: YELLOW");
  OLED.display();
  digitalWrite(YELLOW, LOW);
  delay(1000);
  OLED.clearDisplay();
  OLED.setTextColor(WHITE, BLACK);
  OLED.setCursor(0, 0);
  OLED.setTextSize(1);
  OLED.println("Kitchen Hazard");
  OLED.println("Warning System");
  OLED.setTextColor(BLACK, WHITE);
  OLED.println("Initializing");
  OLED.setTextColor(WHITE, BLACK);
  OLED.println("TEST: GREEN");
  OLED.display();
  digitalWrite(GREEN, LOW);
  delay(1000);
  OLED.clearDisplay();
  OLED.setTextColor(WHITE, BLACK);
  OLED.setCursor(0, 0);
  OLED.setTextSize(1);
  OLED.println("Kitchen Hazard");
  OLED.println("Warning System");
  OLED.setTextColor(BLACK, WHITE);
  OLED.println("Initializing");
  OLED.setTextColor(WHITE, BLACK);
  OLED.println("TEST: ALARM");
  OLED.display();
  digitalWrite(BUZZER, HIGH);
  delay(1000);
  digitalWrite(RED, HIGH);
  digitalWrite(GREEN, HIGH);
  digitalWrite(YELLOW, HIGH);
  digitalWrite(BUZZER, LOW);

  for(int i=30; i>=0; i--){
    OLED.clearDisplay();
    OLED.setTextColor(WHITE, BLACK);
    OLED.setCursor(0, 0);
    OLED.setTextSize(1);
    OLED.println("Kitchen Hazard");
    OLED.println("Warning System");
    OLED.setTextColor(BLACK, WHITE);
    OLED.setTextColor(WHITE, BLACK);
    OLED.println("Warming Gas Sensor: ");
    OLED.print(i, DEC);
    OLED.println(" second(s)");
    OLED.display();
    delay(1000);
  }
  
  /*Setup Global*/
  G_gas_value = 0.00;
  G_alarm_state = 0; // 0 for safe, 1 for alarming, 2 for danger.

  /*Starting USB-Serial Interface*/
  usbInit();
  usbDeviceDisconnect();
  delay(300);
  usbDeviceConnect();

}

void loop() {
  /*Must not take more than 50 ms per iteration*/
  interrupts();
  usbPoll(); /*Polling Client Request*/

  /*Calibration Code -- Uncomment to calibrate*/
//  float sensor_volt;
//  float rs_air;
//  float r0;
//  float sensor_val=0;
//  for(int i=0; i<100; i++) sensor_val += analogRead(GAS);
//  sensor_val /= 100;
//  sensor_volt = (sensor_val/1024) * 5.0;
//  rs_air = (5.0-sensor_volt)/sensor_volt;
//  r0 = rs_air/9.9;
//  
  /*Routine Work*/
  digitalWrite(MB_LED, LOW);
  G_gas_value = get_mq9_reading(GAS);
  
  float calc_gas = G_gas_value/100;
  uint8_t CO=0; /*0 for undetectable*/
  uint8_t LPG=0;
  /*Table CO, LPG concentration*/
  /*Gas concentration relation to rs/r0 is non-linear and datasheet does not provide a formula, only a graph.*/
  /*Phase 
   * Alarm level based on possible CO concentration
   * Phase 1 rs/r0: ~1.8-2.2  CO = <200 ppm; LPG  = 200-400 ppm
   * Phase 2 rs/r0: ~1.5-1.8  CO = 200-300 ppm; LPG = 400-500 ppm /YELLOW ALARM/
   * Phase 3 rs/r0: ~1.0-1.5  CO = 300-500 ppm; LPG = 500-1000 ppm /RED ALARM + BUZZER/
   * Phase 4 rs/r0  ~0.8-1.0  CO = 500-1000 ppm; LPG = 1000-1600 ppm /RED ALERT/
   * Phase 5 rs/r0  ~0.7-0.8  CO = >1000ppm; LPG = 1600-2000 ppm
   * Phase 6 rs/r0  ~0.6-0.7  CO = >1000ppm; LPG = 2000-3000 ppm
   * Phase 7 rs/r0  ~0.4-0.6  CO = >1000ppm; LPG = 3000-8000 ppm
   * Phase 8 rs/r0  ~0.35-0.4 CO = >1000ppm; LPG = 8000-10000ppm
   * Phase 9 rs/r0 <0.35      CO = >1000ppm; LPG = >10000 ppm
  */
  /*table value + observed bias*/
  if(calc_gas > 2.2 + 3){CO = 0; LPG = 0;} //undetectable lower than 200 ppm.
  else if(calc_gas <= 2.2 + 3 && calc_gas > 1.8 + 3){CO =0 ; LPG = 1;}
  else if(calc_gas <= 1.8 + 3 && calc_gas > 1.5 + 3){CO = 1; LPG = 2;}
  else if(calc_gas <= 1.5 + 3 && calc_gas > 1.0 + 3){CO = 2; LPG = 3;}
  else if(calc_gas <= 1.0 + 3 && calc_gas > 0.8 + 3){CO = 3; LPG = 4;}
  else if(calc_gas <= 0.8 + 3 && calc_gas > 0.7 + 3){CO = 4; LPG = 5;}
  else if(calc_gas <= 0.7 + 3 && calc_gas > 0.6 + 3){CO = 4; LPG = 6;}
  else if(calc_gas <= 0.6 + 3 && calc_gas > 0.4 + 3){CO = 4; LPG = 7;}
  else if(calc_gas <= 0.4 + 3 && calc_gas > 0.35 + 3){CO = 4; LPG = 8;}
  else {CO=4; LPG=9;}

  
  OLED.clearDisplay();
  OLED.setTextColor(BLACK, WHITE);
  OLED.setCursor(0, 0);
  OLED.setTextSize(2);

  
//  OLED.println(sensor_volt, DEC);
//  OLED.println(rs_air, DEC);
//  OLED.println(r0, DEC);
//  
  if(G_gas_value >= 700 && G_alarm_state == 0){
    digitalWrite(GREEN, LOW);
    digitalWrite(RED, HIGH);
    digitalWrite(YELLOW, HIGH);
    digitalWrite(BUZZER, LOW);
    OLED.println("  SAFE  ");
//    OLED.println(G_gas_value)
  }
  else if((G_gas_value < 700 && G_gas_value >= 500) || G_alarm_state == 1){
    digitalWrite(YELLOW, LOW);
    digitalWrite(BUZZER, HIGH);
    digitalWrite(RED, HIGH);
    digitalWrite(GREEN, HIGH);
    OLED.println("ALARMING");
  }
  else if(G_gas_value < 500 || G_alarm_state == 2){
    digitalWrite(RED, LOW);
    digitalWrite(BUZZER, HIGH);
    digitalWrite(GREEN, HIGH);
    digitalWrite(YELLOW, HIGH);
    OLED.println("!!DANGER!!");
  }
  OLED.setTextSize(1);
  OLED.setTextColor(WHITE, BLACK);
  OLED.print("CO:  ");
  if(CO == 0){OLED.print("<200");}
  else if(CO == 1){OLED.print("200-300");}
  else if(CO == 2){OLED.print("300-500");}
  else if(CO == 3){OLED.print("500-1000");}
  else{OLED.print(">1000");}
  OLED.println(" ppm");
  OLED.print("LPG: ");
  
  if(LPG == 0){OLED.print("<200");}
  else if(LPG == 1){OLED.print("200-400");}
  else if(LPG == 2){OLED.print("400-500");}
  else if(LPG == 3){OLED.print("500-1000");}
  else if(LPG == 4){OLED.print("1000-1600");}
  else if(LPG == 5){OLED.print("1600-2000");}
  else if(LPG == 6){OLED.print("2000-3000");}
  else if(LPG == 7){OLED.print("3000-8000");}
  else if(LPG == 8){OLED.print("8000-10000");}
  else if(LPG == 9){OLED.print(">10000");}
  OLED.println(" ppm");
  OLED.display();
  digitalWrite(MB_LED, HIGH);
}

