import datetime
from datetime import time
from fastapi import FastAPI, Request
from fastapi.staticfiles import StaticFiles
from fastapi.templating import Jinja2Templates
from starlette.responses import RedirectResponse
from db import Database
from control.usbctrl import Control

db = Database()
device = Control()

def data_parser(data):
    item_time = []
    item_data = []
    for item in data:
        t_item_time = item[0].split(':')
        item_time.append(time(hour=int(t_item_time[0]), minute=int(t_item_time[1]), second=int(t_item_time[2])))
        item_data.append(item[1])
    return item_time, item_data

def create_app():
    db.create()

    app = FastAPI()
    app.mount('/static', StaticFiles(directory="static"), name="static")
    templates = Jinja2Templates(directory="templates")
    print("[APP] CREATED APP")

    @app.get("/")
    async def root(request: Request):
        date = datetime.datetime.today()
        datestring = '{}-{:02d}-{}'.format(date.year, date.month, date.day)
        legend = 'Gas Concentration'
        # Select today data
        data = db.query_noformat(SEL="TIME(timestamp), CO_LPG", COMD=f'WHERE DATE(timestamp) = "{datestring}"')
        labels, values = data_parser(data)
        alarm_level = device.get_alarm()
        return templates.TemplateResponse(
            "index.html", {"request": request,
                           "Day": datestring,
                           "values": values,
                           "labels": labels,
                           "legend": legend,
                           "alarm_level": alarm_level,}
            )

    @app.get("/get_from_date")
    async def get_from_date(request: Request, date: str):
        date = date.split('-')
        datestring = '{}-{:02d}-{}'.format(int(date[0]), int(date[1]), int(date[2]))
        legend = 'Gas Concentration'
        # Select today data
        data = db.query_noformat(SEL="TIME(timestamp), CO_LPG", COMD=f'WHERE DATE(timestamp) = "{datestring}"')
        labels, values = data_parser(data)
        alarm_level = device.get_alarm()
        return templates.TemplateResponse(
            "alt_index.html", {"request": request,
                           "Day": datestring,
                           "values": values,
                           "labels": labels,
                           "legend": legend,}
            )

    @app.get("/get_last_data")
    async def get_last():
        data = db.query(COMD="ORDER BY timestamp DESC LIMIT 1")[0]
        return data

    @app.get("/time_when_gas_equal")
    async def get_equal(level: int = 0):
        data = db.query(COMD=f"WHERE CO_LPG = {level}")
        return {"data": data}

    @app.get("/set_alarm_state")
    async def set_alarm(state: int = 0, return_state: int = 1):
        device.set_alarm(state)
        if return_state:
            return {"new_alarm_state": state}
        else: return RedirectResponse('/')

    @app.get("/get_alarm_state")
    async def get_alarm():
        state = device.get_alarm()
        return {"current_alarm_state": state}

    print("[APP] PATH REGISTERED")

    return app

app_uvicorn = create_app()

if '__main__' == __name__:
    print("[WARNING] DEVELOPMENT MODE")
    app = create_app()
